Class: SC::Buildfile::Task

Inherits:
Object show all
Defined in:
lib/sproutcore/buildfile/task.rb

Overview

Buildfile tasks are rake tasks with a few extras added to support unique buildfile constraints. Much of this source code is borrowed from Rake 0.8.3

Direct Known Subclasses

BuildTask

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(task_name, app) ⇒ Task

Create a task named task_name with no actions or prerequisites. Use enhance to add actions and prerequisites.



73
74
75
76
77
78
79
80
81
82
83
84
# File 'lib/sproutcore/buildfile/task.rb', line 73

def initialize(task_name, app)
  @name = task_name.to_s
  @prerequisites = []
  @actions = []
  @full_comment = nil
  @comment = nil
  @application = app
  @scope = app.current_scope
  @arg_names = nil
  @invoke_count = @execute_count = 0
  @task_options = {}
end

Instance Attribute Details

#actionsObject (readonly)

List of actions attached to a task.



19
20
21
# File 'lib/sproutcore/buildfile/task.rb', line 19

def actions
  @actions
end

#applicationObject

Application owning this task.



22
23
24
# File 'lib/sproutcore/buildfile/task.rb', line 22

def application
  @application
end

#commentObject

Comment for this task. Restricted to a single line of no more than 50 characters.



26
27
28
# File 'lib/sproutcore/buildfile/task.rb', line 26

def comment
  @comment
end

#execute_countObject (readonly)

The number of times the task was actually executed. This may differ from the invoke_count if the task was invoked but was not needed.



40
41
42
# File 'lib/sproutcore/buildfile/task.rb', line 40

def execute_count
  @execute_count
end

#full_commentObject (readonly)

Full text of the (possibly multi-line) comment.



29
30
31
# File 'lib/sproutcore/buildfile/task.rb', line 29

def full_comment
  @full_comment
end

#invoke_countObject (readonly)

The number of times this task has been invoked. Use to ensure that the task was invoked during some call chain…



36
37
38
# File 'lib/sproutcore/buildfile/task.rb', line 36

def invoke_count
  @invoke_count
end

#prerequisitesObject (readonly)

List of prerequisites for a task.



16
17
18
# File 'lib/sproutcore/buildfile/task.rb', line 16

def prerequisites
  @prerequisites
end

#scopeObject (readonly)

Array of nested namespaces names used for task lookup by this task.



32
33
34
# File 'lib/sproutcore/buildfile/task.rb', line 32

def scope
  @scope
end

#sourcesObject



56
57
58
# File 'lib/sproutcore/buildfile/task.rb', line 56

def sources
  @sources ||= []
end

#task_optionsObject (readonly)

Various options you can set on the task to control log level, etc.



43
44
45
# File 'lib/sproutcore/buildfile/task.rb', line 43

def task_options
  @task_options
end

Class Method Details

.indent_logsObject



147
148
149
# File 'lib/sproutcore/buildfile/task.rb', line 147

def self.indent_logs
  @task_indent = (@task_indent || '') + '  '
end

.log_indentObject



143
144
145
# File 'lib/sproutcore/buildfile/task.rb', line 143

def self.log_indent
  @task_indent ||= ''
end

.outdent_logsObject



151
152
153
# File 'lib/sproutcore/buildfile/task.rb', line 151

def self.outdent_logs
  @task_indent = (@task_indent || '')[0..-3]
end

.scope_name(scope, task_name) ⇒ Object

Apply the scope to the task name according to the rules for this kind of task. Generic tasks will accept the scope as part of the name.



315
316
317
# File 'lib/sproutcore/buildfile/task.rb', line 315

def scope_name(scope, task_name)
  (scope + [task_name]).join(':')
end

Instance Method Details

#add_description(description) ⇒ Object

Add a description to the task. The description can consist of an option argument list (enclosed brackets) and an optional comment.



250
251
252
253
254
# File 'lib/sproutcore/buildfile/task.rb', line 250

def add_description(description)
  return if ! description
  comment = description.strip
  add_comment(comment) if comment && ! comment.empty?
end

#add_options(task_options) ⇒ Object

Add task options.



257
258
259
260
# File 'lib/sproutcore/buildfile/task.rb', line 257

def add_options(task_options)
  return if !task_options
  @task_options = task_options
end

#arg_descriptionObject

Argument description (nil if none).



108
109
110
# File 'lib/sproutcore/buildfile/task.rb', line 108

def arg_description # :nodoc:
  @arg_names ? "[#{(arg_names || []).join(',')}]" : nil
end

#arg_namesObject

Name of arguments for this task.



113
114
115
# File 'lib/sproutcore/buildfile/task.rb', line 113

def arg_names
  @arg_names || []
end

#clearObject

Clear the existing prerequisites and actions of a rake task.



118
119
120
121
122
# File 'lib/sproutcore/buildfile/task.rb', line 118

def clear
  clear_prerequisites
  clear_actions
  self
end

#clear_actionsObject

Clear the existing actions on a rake task.



131
132
133
134
# File 'lib/sproutcore/buildfile/task.rb', line 131

def clear_actions
  actions.clear
  self
end

#clear_prerequisitesObject

Clear the existing prerequisites of a rake task.



125
126
127
128
# File 'lib/sproutcore/buildfile/task.rb', line 125

def clear_prerequisites
  prerequisites.clear
  self
end

#dup(app = application) ⇒ Object



65
66
67
68
69
# File 'lib/sproutcore/buildfile/task.rb', line 65

def dup(app=application)
  ret = super()
  ret.application = app
  return ret
end

#enhance(deps = nil, &block) ⇒ Object

Enhance a task with prerequisites or actions. Returns self.



87
88
89
90
91
# File 'lib/sproutcore/buildfile/task.rb', line 87

def enhance(deps=nil, &block)
  @prerequisites |= deps if deps
  @actions << block if block_given?
  self
end

#execute(args = nil) ⇒ Object

Execute the actions associated with this task.



221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
# File 'lib/sproutcore/buildfile/task.rb', line 221

def execute(args=nil)
  @execute_count += 1
  args ||= EMPTY_TASK_ARGS
  return if SC.env[:dryrun]

  @actions.each do |act|
    case act.arity
    when 1
      act.call(self)
    else
      act.call(self, args)
    end
  end

end

#inspectObject



50
51
52
# File 'lib/sproutcore/buildfile/task.rb', line 50

def inspect
  "<#{self.class} #{name} => [#{prerequisites.join(', ')}]>"
end

#investigationObject

Return a string describing the internal state of a task. Useful for debugging.



292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
# File 'lib/sproutcore/buildfile/task.rb', line 292

def investigation
  result = "------------------------------\n"
  result << "Investigating #{name}\n"
  result << "class: #{self.class}\n"
  result <<  "task needed: #{needed?}\n"
  result <<  "timestamp: #{timestamp}\n"
  result << "pre-requisites: \n"
  prereqs = @prerequisites.collect {|name| application[name]}
  prereqs.sort! {|a,b| a.timestamp <=> b.timestamp}
  prereqs.each do |p|
    result << "--#{p.name} (#{p.timestamp})\n"
  end
  latest_prereq = @prerequisites.collect{|n| application[n].timestamp}.max
  result <<  "latest-prerequisite time: #{latest_prereq}\n"
  result << "................................\n\n"
  return result
end

#invoke(hash) ⇒ Object

Invoke the task if it is needed. Prerequites are invoked first.



137
138
139
140
141
# File 'lib/sproutcore/buildfile/task.rb', line 137

def invoke(hash)
  task_args = hash
  task_args = TaskArguments.with_hash(hash) #TaskArguments.new(arg_names, args)
  invoke_with_call_chain(task_args, InvocationChain::EMPTY)
end

#invoke_prerequisites(task_args, invocation_chain) ⇒ Object

Invoke all the prerequisites of a task.



202
203
204
205
206
207
208
209
# File 'lib/sproutcore/buildfile/task.rb', line 202

def invoke_prerequisites(task_args, invocation_chain) # :nodoc:
  @prerequisites.each { |n|
    prereq = application[n, @scope]
    prereq_args = task_args.new_scope(prereq.arg_names)
    invocation_chain = prereq.invoke_with_call_chain(prereq_args, invocation_chain)
  }
  return invocation_chain
end

#nameObject

Name of the task, including any namespace qualifiers.



94
95
96
# File 'lib/sproutcore/buildfile/task.rb', line 94

def name
  @name.to_s
end

#name_with_argsObject

Name of task with argument list description.



99
100
101
102
103
104
105
# File 'lib/sproutcore/buildfile/task.rb', line 99

def name_with_args # :nodoc:
  if arg_description
    "#{name}#{arg_description}"
  else
    name
  end
end

#needed?Boolean

Is this task needed?

Returns:



238
239
240
# File 'lib/sproutcore/buildfile/task.rb', line 238

def needed?
  true
end

#set_arg_names(args) ⇒ Object

Set the names of the arguments for this task. args should be an array of symbols, one for each argument name.



286
287
288
# File 'lib/sproutcore/buildfile/task.rb', line 286

def set_arg_names(args)
  @arg_names = args.map { |a| a.to_sym }
end

#sourceObject

First source from a rule (nil if no sources)



61
62
63
# File 'lib/sproutcore/buildfile/task.rb', line 61

def source
  @sources.first if defined?(@sources)
end

#timestampObject

Timestamp for this task. Basic tasks return the current time for their time stamp. Other tasks can be more sophisticated.



244
245
246
# File 'lib/sproutcore/buildfile/task.rb', line 244

def timestamp
  @prerequisites.collect { |p| application[p].timestamp }.max || Time.now
end

#to_sObject

Return task name



46
47
48
# File 'lib/sproutcore/buildfile/task.rb', line 46

def to_s
  name
end